@@ -4,7 +4,7 @@
  * @licstart  The following is the entire license notice for the
  * JavaScript code in this file.
  *
- * Copyright (c) 2012-2014, The Roundcube Dev Team
+ * Copyright (c) The Roundcube Dev Team
  *
  * The JavaScript code in this page is free software: you can redistribute it
  * and/or modify it under the terms of the GNU General Public License
@@ -56,7 +56,8 @@ if (window.rcmail) {
         sieve_raw_editor_init();
       }
       else {
-        rcmail.enable_command('plugin.managesieve-add', 'plugin.managesieve-setadd', !rcmail.env.sieveconnerror);
+        rcmail.enable_command('plugin.managesieve-add', !rcmail.env.sieveconnerror && $.inArray('new_rule', rcmail.env.managesieve_disabled_actions) == -1);
+        rcmail.enable_command('plugin.managesieve-setadd', !rcmail.env.sieveconnerror && $.inArray('new_set', rcmail.env.managesieve_disabled_actions) == -1);
       }
 
       var setcnt, set = rcmail.env.currentset;
@@ -66,9 +67,10 @@ if (window.rcmail) {
           {multiselect:false, draggable:true, keyboard:true});
 
         rcmail.filters_list
-          .addEventListener('select', function(e) { rcmail.managesieve_select(e); })
-          .addEventListener('dragstart', function(e) { rcmail.managesieve_dragstart(e); })
-          .addEventListener('dragend', function(e) { rcmail.managesieve_dragend(e); })
+          .addEventListener('select', function(o) { rcmail.managesieve_select(o); })
+          .addEventListener('keypress', function(o) { rcmail.list_keypress(o, {del: 'plugin.managesieve-del'}); })
+          .addEventListener('dragstart', function(o) { rcmail.managesieve_dragstart(o); })
+          .addEventListener('dragend', function(o) { rcmail.managesieve_dragend(o); })
           .addEventListener('initrow', function(row) {
             row.obj.onmouseover = function() { rcmail.managesieve_focus_filter(row); };
             row.obj.onmouseout = function() { rcmail.managesieve_unfocus_filter(row); };
@@ -83,6 +85,7 @@ if (window.rcmail) {
         rcmail.filtersets_list.init().focus();
 
         if (set != null) {
+          $('#filterset-name').text(set);
           set = rcmail.managesieve_setid(set);
           rcmail.filtersets_list.select(set);
         }
@@ -92,8 +95,9 @@ if (window.rcmail) {
 
         setcnt = rcmail.filtersets_list.rowcount;
         rcmail.enable_command('plugin.managesieve-set', true);
-        rcmail.enable_command('plugin.managesieve-setact', 'plugin.managesieve-setget', setcnt > 0);
-        rcmail.enable_command('plugin.managesieve-setdel', setcnt > 1);
+        rcmail.enable_command('plugin.managesieve-setact', setcnt > 0 && $.inArray('enable_disable_set', rcmail.env.managesieve_disabled_actions) == -1);
+        rcmail.enable_command('plugin.managesieve-setget', setcnt > 0 && $.inArray('download_set', rcmail.env.managesieve_disabled_actions) == -1);
+        rcmail.enable_command('plugin.managesieve-setdel', setcnt > 1 && $.inArray('delete_set', rcmail.env.managesieve_disabled_actions) == -1);
         rcmail.enable_command('plugin.managesieve-seteditraw', setcnt > 0 && rcmail.env.raw_sieve_editor);
 
         // Fix dragging filters over sets list
@@ -112,17 +116,18 @@ if (window.rcmail) {
 
 rcube_webmail.prototype.managesieve_add = function()
 {
-  this.load_managesieveframe('', true);
+  this.load_managesieveframe('_nav=hide', true);
 };
 
 rcube_webmail.prototype.managesieve_del = function()
 {
   var id = this.filters_list.get_single_selection();
-  if (confirm(this.get_label('managesieve.filterdeleteconfirm'))) {
-    var lock = this.set_busy(true, 'loading');
-    this.http_post('plugin.managesieve-action',
-      '_act=delete&_fid='+this.filters_list.rows[id].uid, lock);
-  }
+  this.confirm_dialog(this.get_label('managesieve.filterdeleteconfirm'), 'delete', function(e, ref) {
+      var post = '_act=delete&_fid=' + ref.filters_list.rows[id].uid,
+        lock = ref.set_busy(true, 'loading');
+
+      ref.http_post('plugin.managesieve-action', post, lock);
+    });
 };
 
 rcube_webmail.prototype.managesieve_act = function()
@@ -145,21 +150,30 @@ rcube_webmail.prototype.managesieve_select = function(list)
   }
 
   var has_id = typeof(id) != 'undefined' && id != null;
-  this.enable_command('plugin.managesieve-act', 'plugin.managesieve-del', has_id);
+
+  this.enable_command('plugin.managesieve-act', has_id);
+  this.enable_command('plugin.managesieve-del', has_id && $.inArray('delete_rule', rcmail.env.managesieve_disabled_actions) == -1);
 };
 
 // Set selection
 rcube_webmail.prototype.managesieve_setselect = function(list)
 {
-  this.show_contentframe(false);
-  this.filters_list.clear(true);
-  this.enable_command('plugin.managesieve-setdel', list.rowcount > 1);
-  this.enable_command('plugin.managesieve-setact', 'plugin.managesieve-setget', list.rowcount > 0);
+  this.enable_command('plugin.managesieve-setdel', list.rowcount > 1 && $.inArray('delete_set', rcmail.env.managesieve_disabled_actions) == -1);
+  this.enable_command('plugin.managesieve-setact', list.rowcount > 0 && $.inArray('enable_disable_set', rcmail.env.managesieve_disabled_actions) == -1);
+  this.enable_command('plugin.managesieve-setget', list.rowcount > 0 && $.inArray('delete_set', rcmail.env.managesieve_disabled_actions) == -1);
   this.enable_command('plugin.managesieve-seteditraw', list.rowcount > 0 && this.env.raw_sieve_editor);
 
+ if (rcmail.env.contextmenu_opening)
+   return;
+
+  this.show_contentframe(false);
+  this.filters_list.clear(true);
+
   var id = list.get_single_selection();
-  if (id != null)
+  if (id != null) {
     this.managesieve_list(this.env.filtersets[id]);
+    $('#filterset-name').text(this.env.filtersets[id]);
+  }
 };
 
 rcube_webmail.prototype.managesieve_rowid = function(id)
@@ -210,14 +224,13 @@ rcube_webmail.prototype.managesieve_setact = function()
 // Set delete request
 rcube_webmail.prototype.managesieve_setdel = function()
 {
-  if (!confirm(this.get_label('managesieve.setdeleteconfirm')))
-    return false;
+  var id = this.filtersets_list.get_single_selection();
+  this.confirm_dialog(this.get_label('managesieve.setdeleteconfirm'), 'delete', function(e, ref) {
+      var script = ref.env.filtersets[id],
+        lock = ref.set_busy(true, 'loading');
 
-  var id = this.filtersets_list.get_single_selection(),
-    lock = this.set_busy(true, 'loading'),
-    script = this.env.filtersets[id];
-
-  this.http_post('plugin.managesieve-action', '_act=setdel&_set='+urlencode(script), lock);
+      ref.http_post('plugin.managesieve-action', '_act=setdel&_set=' + urlencode(script), lock);
+    });
 };
 
 // Set edit raw request
@@ -226,13 +239,13 @@ rcube_webmail.prototype.managesieve_seteditraw = function()
   var id = this.filtersets_list.get_single_selection(),
     script = this.env.filtersets[id];
 
-  this.load_managesieveframe('_seteditraw=1&_set=' + urlencode(script), true);
-}
+  this.load_managesieveframe('_nav=hide&_seteditraw=1&_set=' + urlencode(script), true);
+};
 
 // Set add request
 rcube_webmail.prototype.managesieve_setadd = function()
 {
-  this.load_managesieveframe('_newset=1', true);
+  this.load_managesieveframe('_nav=hide&_newset=1', true);
 };
 
 rcube_webmail.prototype.managesieve_updatelist = function(action, o)
@@ -299,7 +312,8 @@ rcube_webmail.prototype.managesieve_updatelist = function(action, o)
       list.insert_row(row.get(0));
       list.highlight_row(o.id);
 
-      this.enable_command('plugin.managesieve-del', 'plugin.managesieve-act', true);
+      this.enable_command('plugin.managesieve-del', $.inArray('delete_rule', rcmail.env.managesieve_disabled_actions) == -1);
+      this.enable_command('plugin.managesieve-act', true);
 
       break;
 
@@ -349,13 +363,13 @@ rcube_webmail.prototype.managesieve_updatelist = function(action, o)
     case 'setdel':
       var id = this.managesieve_setid(o.name);
 
-      this.filtersets_list.remove_row(id);
       this.filters_list.clear();
       this.show_contentframe(false);
-      this.enable_command('plugin.managesieve-setdel', 'plugin.managesieve-setact', 'plugin.managesieve-setget', false);
+      this.enable_command('plugin.managesieve-setdel', 'plugin.managesieve-setact',
+        'plugin.managesieve-setget', 'plugin.managesieve-seteditraw', false);
 
+      this.filtersets_list.remove_row(id, true);
       delete this.env.filtersets[id];
-
       break;
 
     // Create set row
@@ -413,9 +427,9 @@ rcube_webmail.prototype.load_managesieveframe = function(add_url, reset)
     this.reset_filters_list();
 
   if (this.env.contentframe && window.frames && window.frames[this.env.contentframe]) {
-    var lock = this.set_busy(true, 'loading');
+    var lock = this.set_busy(true, 'loading'),
+      target = window.frames[this.env.contentframe];
 
-    target = window.frames[this.env.contentframe];
     target.location.href = this.env.comm_path
       + '&_action=plugin.managesieve-action&_framed=1&_unlock=' + lock
       + (add_url ? ('&' + add_url) : '');
@@ -486,6 +500,12 @@ rcube_webmail.prototype.managesieve_save = function()
     return;
   }
 
+  if (this.env.action == 'plugin.managesieve-forward') {
+    var data = $(this.gui_objects.sieveform).serialize();
+    this.http_post('plugin.managesieve-forward', data, this.display_message(this.get_label('managesieve.forward.saving'), 'loading'));
+    return;
+  }
+
   if (this.gui_objects.sieveform) {
     if (parent.rcmail && parent.rcmail.filters_list && this.gui_objects.sieveform.name != 'filtersetform') {
       var id = parent.rcmail.filters_list.get_single_selection();
@@ -509,14 +529,11 @@ rcube_webmail.prototype.managesieve_rulefill = function(content, id, after)
 {
   if (content != '') {
     // create new element
-    var div = document.getElementById('rules'),
-      row = document.createElement('div');
+    var div = $('#rules')[0],
+      row = $('<div>').attr({'class': 'rulerow', id: 'rulerow'+id})
+        .html(content);
 
     this.managesieve_insertrow(div, row, after);
-    // fill row after inserting (for IE)
-    row.setAttribute('id', 'rulerow'+id);
-    row.className = 'rulerow';
-    row.innerHTML = content;
 
     // initialize smart list inputs
     $('textarea[data-type="list"]', row).each(function() {
@@ -532,11 +549,11 @@ rcube_webmail.prototype.managesieve_ruledel = function(id)
   if ($('#ruledel'+id).hasClass('disabled'))
     return;
 
-  if (confirm(this.get_label('managesieve.ruledeleteconfirm'))) {
-    var row = document.getElementById('rulerow'+id);
-    row.parentNode.removeChild(row);
-    this.managesieve_formbuttons(document.getElementById('rules'));
-  }
+  this.confirm_dialog(this.get_label('managesieve.ruledeleteconfirm'), 'delete', function(e, ref) {
+      var row = document.getElementById('rulerow'+id);
+      row.parentNode.removeChild(row);
+      ref.managesieve_formbuttons(document.getElementById('rules'));
+    });
 };
 
 rcube_webmail.prototype.managesieve_actionadd = function(id)
@@ -547,14 +564,11 @@ rcube_webmail.prototype.managesieve_actionadd = function(id)
 rcube_webmail.prototype.managesieve_actionfill = function(content, id, after)
 {
   if (content != '') {
-    var div = document.getElementById('actions'),
-      row = document.createElement('div');
+    var div = $('#actions')[0],
+      row = $('<div>').attr({'class': 'actionrow', id: 'actionrow'+id})
+        .html(content);
 
     this.managesieve_insertrow(div, row, after);
-    // fill row after inserting (for IE)
-    row.className = 'actionrow';
-    row.setAttribute('id', 'actionrow'+id);
-    row.innerHTML = content;
 
     // initialize smart list inputs
     $('textarea[data-type="list"]', row).each(function() {
@@ -570,53 +584,34 @@ rcube_webmail.prototype.managesieve_actiondel = function(id)
   if ($('#actiondel'+id).hasClass('disabled'))
     return;
 
-  if (confirm(this.get_label('managesieve.actiondeleteconfirm'))) {
-    var row = document.getElementById('actionrow'+id);
-    row.parentNode.removeChild(row);
-    this.managesieve_formbuttons(document.getElementById('actions'));
-  }
+  this.confirm_dialog(this.get_label('managesieve.actiondeleteconfirm'), 'delete', function(e, ref) {
+      var row = document.getElementById('actionrow'+id);
+      row.parentNode.removeChild(row);
+      ref.managesieve_formbuttons(document.getElementById('actions'));
+    });
 };
 
 // insert rule/action row in specified place on the list
 rcube_webmail.prototype.managesieve_insertrow = function(div, row, after)
 {
-  for (var i=0; i<div.childNodes.length; i++) {
-    if (div.childNodes[i].id == (div.id == 'rules' ? 'rulerow' : 'actionrow')  + after)
-      break;
-  }
+  var node = $('#' + ($(div).attr('id') == 'rules' ? 'rulerow' : 'actionrow')  + after)[0];
 
-  if (div.childNodes[i+1])
-    div.insertBefore(row, div.childNodes[i+1]);
+  if (node)
+    $(row).insertAfter(node);
   else
-    div.appendChild(row);
+    $(div).append(row);
+
+  this.triggerEvent('managesieve.insertrow', {obj: row});
 };
 
 // update Delete buttons status
 rcube_webmail.prototype.managesieve_formbuttons = function(div)
 {
-  var i, button, buttons = [];
+  var buttons = $('a.delete', div);
 
-  // count and get buttons
-  for (i=0; i<div.childNodes.length; i++) {
-    if (div.id == 'rules' && div.childNodes[i].id) {
-      if (/rulerow/.test(div.childNodes[i].id))
-        buttons.push('ruledel' + div.childNodes[i].id.replace(/rulerow/, ''));
-    }
-    else if (div.childNodes[i].id) {
-      if (/actionrow/.test(div.childNodes[i].id))
-        buttons.push( 'actiondel' + div.childNodes[i].id.replace(/actionrow/, ''));
-    }
-  }
-
-  for (i=0; i<buttons.length; i++) {
-    button = document.getElementById(buttons[i]);
-    if (i>0 || buttons.length>1) {
-      $(button).removeClass('disabled');
-    }
-    else {
-      $(button).addClass('disabled');
-    }
-  }
+  buttons.removeClass('disabled');
+  if (buttons.length == 1)
+    buttons.addClass('disabled');
 };
 
 // update vacation addresses field with user identities
@@ -635,7 +630,8 @@ rcube_webmail.prototype.managesieve_vacation_addresses_update = function(id, add
 
 function rule_header_select(id)
 {
-  var obj = document.getElementById('header' + id),
+  var is_header,
+    obj = document.getElementById('header' + id),
     size = document.getElementById('rule_size' + id),
     msg = document.getElementById('rule_message' + id),
     op = document.getElementById('rule_op' + id),
@@ -644,31 +640,38 @@ function rule_header_select(id)
     mod = document.getElementById('rule_mod' + id),
     trans = document.getElementById('rule_trans' + id),
     comp = document.getElementById('rule_comp' + id),
+    mime = document.getElementById('rule_mime' + id),
+    mime_part = document.getElementById('rule_mime_part' + id),
     datepart = document.getElementById('rule_date_part' + id),
     dateheader = document.getElementById('rule_date_header_div' + id),
     rule = $('#rule_op' + id),
     h = obj.value,
-    set = [op, header, custstr, mod, trans, comp, size];
+    set = [op, header, custstr, mod, trans, comp, size, mime, mime_part];
 
   if (h == 'size') {
     if (msg) set.push(msg);
-    $.each(set, function() { this.style.display = 'none'; });
-    size.style.display = 'inline';
+    $.each(set, function() { if (this != window) this.style.display = 'none'; });
+    size.style.display = '';
   }
   else if (h == 'message' && msg) {
-    $.each(set, function() { this.style.display = 'none'; });
-    msg.style.display = 'inline';
+    $.each(set, function() { if (this != window)  this.style.display = 'none'; });
+    msg.style.display = '';
   }
   else {
-    header.style.display = h != '...' ? 'none' : 'inline-block';
-    custstr.style.display = h != 'string' ? 'none' : 'inline-block';
+    is_header = h != 'body' && h != 'currentdate' && h != 'date' && h != 'string';
+    header.style.display = h != '...' ? 'none' : '';
+    custstr.style.display = h != 'string' ? 'none' : '';
     size.style.display = 'none';
-    op.style.display = 'inline';
+    op.style.display = '';
     comp.style.display = '';
-    mod.style.display = h == 'body' || h == 'currentdate' || h == 'date' || h == 'string' ? 'none' : 'block';
-    trans.style.display = h == 'body' ? 'block' : 'none';
+    mod.style.display = is_header ? '' : 'none';
+    trans.style.display = h == 'body' ? '' : 'none';
+    if (mime)
+      mime.style.display =  is_header ? '' : 'none';
+    if (mime_part)
+      mime_part.style.display = is_header ? '' : 'none';
     if (msg)
-      msg.style.display = h == 'message' ? 'block' : 'none';
+      msg.style.display = h == 'message' ? '' : 'none';
   }
 
   if (datepart)
@@ -677,11 +680,12 @@ function rule_header_select(id)
     dateheader.style.display = h == 'date' ? '' : 'none';
 
   $('[value="exists"],[value="notexists"]', rule).prop('disabled', h == 'string');
-  if (!rule.val() || rule.val().match(/^(exists|notexists)$/))
+  if (!rule.val())
     rule.val('contains');
 
   rule_op_select(op, id, h);
   rule_mod_select(id, h);
+  rule_mime_select(id);
 
   obj.style.width = h == '...' ? '40px' : '';
 };
@@ -693,7 +697,7 @@ function rule_op_select(obj, id, header)
   if (!header)
     header = document.getElementById('header' + id).value;
 
-  target.style.display = obj.value.match(/^(exists|notexists)$/) || header.match(/^(size|message)$/) ? 'none' : 'inline-block';
+  target.style.display = obj.value.match(/^(exists|notexists)$/) || header.match(/^(size|message)$/) ? 'none' : '';
 };
 
 function rule_trans_select(id)
@@ -714,7 +718,7 @@ function rule_mod_select(id, header)
   if (!header)
     header = document.getElementById('header' + id).value;
 
-  target.style.display = obj.value != 'address' && obj.value != 'envelope' ? 'none' : 'inline';
+  target.style.display = obj.value != 'address' && obj.value != 'envelope' ? 'none' : '';
 
   if (index)
     index.style.display = !header.match(/^(body|currentdate|size|message|string)$/) && obj.value != 'envelope'  ? '' : 'none';
@@ -733,14 +737,23 @@ function rule_adv_switch(id, elem)
   var elem = $(elem), enabled = elem.hasClass('hide'), adv = $('#rule_advanced'+id);
 
   if (enabled) {
-    adv.hide();
+    adv.get(0).style.display = 'none';
     elem.removeClass('hide').addClass('show');
   }
   else {
-    adv.show();
+    adv.get(0).style.display = '';
     elem.removeClass('show').addClass('hide');
   }
-}
+};
+
+function rule_mime_select(id)
+{
+  var elem = $('#rule_mime_type' + id),
+    param_elem = $('#rule_mime_param' + id + '_list');
+
+  if (param_elem.length)
+    param_elem[0].style.display = elem.val() == 'param' ? '' : 'none';
+};
 
 function action_type_select(id)
 {
@@ -752,8 +765,11 @@ function action_type_select(id)
       target_area: document.getElementById('action_target_area' + id),
       flags: document.getElementById('action_flags' + id),
       vacation: document.getElementById('action_vacation' + id),
+      forward: document.getElementById('action_forward' + id),
       set: document.getElementById('action_set' + id),
-      notify: document.getElementById('action_notify' + id)
+      notify: document.getElementById('action_notify' + id),
+      addheader: document.getElementById('action_addheader' + id),
+      deleteheader: document.getElementById('action_deleteheader' + id)
     };
 
   if (v == 'fileinto' || v == 'fileinto_copy') {
@@ -768,18 +784,13 @@ function action_type_select(id)
   else if (v.match(/^(add|set|remove)flag$/)) {
     enabled.flags = 1;
   }
-  else if (v == 'vacation') {
-    enabled.vacation = 1;
-  }
-  else if (v == 'set') {
-    enabled.set = 1;
-  }
-  else if (v == 'notify') {
-    enabled.notify = 1;
+  else if (v.match(/^(vacation|forward|set|notify|addheader|deleteheader)$/)) {
+    enabled[v] = 1;
   }
 
   for (var x in elems) {
-    elems[x].style.display = !enabled[x] ? 'none' : 'inline';
+    if (elems[x])
+      elems[x].style.display = !enabled[x] ? 'none' : 'inline';
   }
 };
 
@@ -793,6 +804,9 @@ function vacation_action_select()
 // Inititalizes smart list input
 function smart_field_init(field)
 {
+  if (window.UI && UI.smart_field_init)
+    return UI.smart_field_init(field);
+
   var id = field.id + '_list',
     area = $('<span class="listarea"></span>'),
     list = field.value ? field.value.split("\n") : [''];
@@ -814,11 +828,14 @@ function smart_field_init(field)
   else
     field.prop('disabled', true);
 
+  if (field.data('hidden'))
+    area.hide();
+
   field.after(area);
 
   if (field.hasClass('error')) {
     area.addClass('error');
-    rcmail.managesieve_tip_register([[id, field.data('tip')]]);
+    rcmail.managesieve_tip_register([[id, field.data('tip-class'), field.data('tip-msg')]]);
   }
 };
 
@@ -878,6 +895,9 @@ function smart_field_row(value, name, idx, size)
 // Reset and fill the smart list input with new data
 function smart_field_reset(field, data)
 {
+  if (window.UI && UI.smart_field_reset)
+    return UI.smart_field_reset(field, data);
+
   var id = field.id + '_list',
     list = data.length ? data : [''];
     area = $('#' + id);
@@ -893,19 +913,24 @@ function smart_field_reset(field, data)
 // Register onmouse(leave/enter) events for tips on specified form element
 rcube_webmail.prototype.managesieve_tip_register = function(tips)
 {
+  if (window.UI && UI.form_errors)
+    return UI.form_errors(tips);
+
   var n, framed = parent.rcmail,
     tip = framed ? parent.rcmail.env.ms_tip_layer : rcmail.env.ms_tip_layer;
 
   for (n in tips) {
     $('#'+tips[n][0])
-      .data('tip', tips[n][1])
+      .data('tip-class', tips[n][1])
+      .data('tip-msg', tips[n][2])
       .mouseleave(function(e) { tip.hide(); })
       .mouseenter(function(e) {
         var elem = $(this),
           offset = elem.offset(),
           left = offset.left,
           top = offset.top - 12,
-          minwidth = elem.width();
+          minwidth = elem.width(),
+          span = $('<span>').addClass(elem.data('tip-class')).text(elem.data('tip-msg'));
 
         if (framed) {
           offset = $((rcmail.env.task == 'mail'  ? '#sievefilterform > iframe' : '#filter-box'), parent.document).offset();
@@ -913,7 +938,7 @@ rcube_webmail.prototype.managesieve_tip_register = function(tips)
           left += offset.left;
         }
 
-        tip.html(elem.data('tip'));
+        tip.html('').append(span);
         top -= tip.height();
 
         tip.css({left: left, top: top, minWidth: (minwidth-2) + 'px'}).show();
@@ -929,8 +954,8 @@ function sieve_formattime(hour, minutes)
   for (i=0; i<format.length; i++) {
     c = format.charAt(i);
     switch (c) {
-      case 'a': time += hour > 12 ? 'am' : 'pm'; break;
-      case 'A': time += hour > 12 ? 'AM' : 'PM'; break;
+      case 'a': time += hour >= 12 ? 'pm' : 'am'; break;
+      case 'A': time += hour >= 12 ? 'PM' : 'AM'; break;
       case 'g':
       case 'h':
         h = hour == 0 ? 12 : hour > 12 ? hour - 12 : hour;
@@ -956,7 +981,7 @@ function sieve_form_init()
     parent.rcmail.managesieve_dialog_resize(form);
   }
 
-  $('input[type="text"]:first', form).focus();
+  $('input[type="text"]', form).first().focus();
 
   // initialize smart list inputs
   $('textarea[data-type="list"]', form).each(function() {
@@ -964,7 +989,7 @@ function sieve_form_init()
   });
 
   // initialize rules form(s)
-  $('[name="_header[]"]', form).each(function() {
+  $('[name^="_header"]', form).each(function() {
     if (/([0-9]+)$/.test(this.id)) {
       rule_header_select(RegExp.$1);
     }
@@ -1091,29 +1116,34 @@ rcube_webmail.prototype.managesieve_create = function(force)
   if (!this.env.sieve_headers || !this.env.sieve_headers.length)
     return;
 
-  var i, html, buttons = {}, dialog = $("#sievefilterform");
+  var i, buttons = {},
+    title = this.get_label('managesieve.newfilter'),
+    dialog = $('<div id="sievefilterform" class="propform"></div>'),
+    props = {minWidth: 600, minHeight: 250, height: 300};
 
-  // create dialog window
-  if (!dialog.length) {
-    dialog = $('<div id="sievefilterform"></div>');
-    $('body').append(dialog);
-  }
 
   // build dialog window content
-  html = '<fieldset><legend>'+this.get_label('managesieve.usedata')+'</legend><ul>';
-  for (i in this.env.sieve_headers)
-    html += '<li><input type="checkbox" name="headers[]" id="sievehdr'+i+'" value="'+i+'" checked="checked" />'
-      +'<label for="sievehdr'+i+'">'+this.env.sieve_headers[i][0]+':</label> '+this.env.sieve_headers[i][1]+'</li>';
-  html += '</ul></fieldset>';
+  dialog.append($('<fieldset>')
+    .append($('<legend>').text(this.get_label('managesieve.usedata')))
+    .append($('<ul class="proplist">'))
+  );
 
-  dialog.html(html);
+  $.each(this.env.sieve_headers, function(i, v) {
+    var attr = {type: 'checkbox', name: 'headers[]', id: 'sievehdr' + i, value: i,  checked: true},
+      label = rcmail.env.sieve_headers[i][0] + ': ' + rcmail.env.sieve_headers[i][1];
+
+    $('ul', dialog).append($('<li>')
+      .append($('<input>').attr(attr))
+      .append($('<label>').attr('for', 'sievehdr' + i).text(label))
+    );
+  });
 
   // [Next Step] button action
   buttons[this.get_label('managesieve.nextstep')] = function () {
     // check if there's at least one checkbox checked
     var hdrs = $('input[name="headers[]"]:checked', dialog);
     if (!hdrs.length) {
-      alert(rcmail.get_label('managesieve.nodata'));
+      rcmail.alert_dialog(rcmail.get_label('managesieve.nodata'));
       return;
     }
 
@@ -1128,50 +1158,44 @@ rcube_webmail.prototype.managesieve_create = function(force)
     });
 
     // load form in the iframe
-    var frame = $('<iframe>').attr({src: url, frameborder: 0})
-    dialog.empty().append(frame).dialog('widget').resize();
+    var buttons = {}, iframe = $('<iframe>').attr({src: url, frameborder: 0});
 
     // Change [Next Step] button with [Save] button
-    buttons = {};
     buttons[rcmail.get_label('save')] = function() {
       var win = $('iframe', dialog).get(0).contentWindow;
       win.rcmail.managesieve_save();
     };
-    dialog.dialog('option', 'buttons', buttons);
+    buttons[rcmail.get_label('cancel')] = function() { $(this).dialog('destroy'); };
+
+    dialog.dialog('destroy');
+
+    rcmail.env.managesieve_dialog = dialog = rcmail.show_popup_dialog(
+      iframe, title, buttons, $.extend(props, {button_classes: ['mainaction save', 'cancel']})
+    );
   };
 
-  // show dialog window
-  dialog.dialog({
-    modal: false,
-    resizable: true,
-    closeOnEscape: true,
-    title: this.get_label('managesieve.newfilter'),
-    close: function() { rcmail.managesieve_dialog_close(); },
-    buttons: buttons,
-    minWidth: 600,
-    minHeight: 300,
-    height: 250
-  }).show();
+  buttons[this.get_label('cancel')] = function() { $(this).dialog('destroy'); };
 
-  this.env.managesieve_dialog = dialog;
+  this.env.managesieve_dialog = dialog = this.show_popup_dialog(
+    dialog, title, buttons, $.extend(props, {button_classes: ['mainaction next', 'cancel']})
+  );
 }
 
 rcube_webmail.prototype.managesieve_dialog_close = function()
 {
-  var dialog = this.env.managesieve_dialog;
-
-  // BUG(?): if we don't remove the iframe first, it will be reloaded
-  dialog.html('');
-  dialog.dialog('destroy').hide();
+  this.env.managesieve_dialog.dialog('destroy');
 }
 
 rcube_webmail.prototype.managesieve_dialog_resize = function(o)
 {
   var dialog = this.env.managesieve_dialog,
     win = $(window), form = $(o);
-    width = $('fieldset:first', o).width(), // fieldset width is more appropriate here
+    width = $('fieldset', o).first().width(), // fieldset width is more appropriate here
     height = form.height(),
     w = win.width(), h = win.height();
 
+  if (height < 100)
+    return;
+
   dialog.dialog('option', { height: Math.min(h-20, height+120), width: Math.min(w-20, width+65) });
 }
