@@ -2,11 +2,10 @@
 
 /**
  +-----------------------------------------------------------------------+
- | program/include/rcmail.php                                            |
- |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2014, The Roundcube Dev Team                       |
- | Copyright (C) 2011-2014, Kolab Systems AG                             |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
+ | Copyright (C) Kolab Systems AG                                        |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -103,6 +102,9 @@ class rcmail extends rcube
         // start session
         $this->session_init();
 
+        // Remember default skin, before it's replaced by user prefs
+        $this->default_skin = $this->config->get('skin');
+
         // create user object
         $this->set_user(new rcube_user($_SESSION['user_id']));
 
@@ -304,13 +306,13 @@ class rcmail extends rcube
      */
     public function get_address_sources($writeable = false, $skip_hidden = false)
     {
-        $abook_type   = (string) $this->config->get('address_book_type');
+        $abook_type   = strtolower((string) $this->config->get('address_book_type', 'sql'));
         $ldap_config  = (array) $this->config->get('ldap_public');
         $autocomplete = (array) $this->config->get('autocomplete_addressbooks');
         $list         = array();
 
-        // We are using the DB address book or a plugin address book
-        if (!empty($abook_type) && strtolower($abook_type) != 'ldap') {
+        // SQL-based (built-in) address book
+        if ($abook_type === 'sql') {
             if (!isset($this->address_books['0'])) {
                 $this->address_books['0'] = new rcube_contacts($this->db, $this->get_user_id());
             }
@@ -321,10 +323,11 @@ class rcmail extends rcube
                 'groups'   => $this->address_books['0']->groups,
                 'readonly' => $this->address_books['0']->readonly,
                 'undelete' => $this->address_books['0']->undelete && $this->config->get('undo_timeout'),
-                'autocomplete' => in_array('sql', $autocomplete),
+                'autocomplete' => in_array_nocase('sql', $autocomplete),
             );
         }
 
+        // LDAP address book(s)
         if (!empty($ldap_config)) {
             foreach ($ldap_config as $id => $prop) {
                 // handle misconfiguration
@@ -343,6 +346,7 @@ class rcmail extends rcube
             }
         }
 
+        // Plugins can also add address books, or re-order the list
         $plugin = $this->plugins->exec_hook('addressbooks_list', array('sources' => $list));
         $list   = $plugin['sources'];
 
@@ -455,7 +459,8 @@ class rcmail extends rcube
 
         // add some basic labels to client
         $this->output->add_label('loading', 'servererror', 'connerror', 'requesttimedout',
-            'refreshing', 'windowopenerror', 'uploadingmany', 'close');
+            'refreshing', 'windowopenerror', 'uploadingmany', 'uploading', 'close', 'save', 'cancel',
+            'alerttitle', 'confirmationtitle', 'delete', 'continue', 'ok');
 
         return $this->output;
     }
@@ -485,11 +490,6 @@ class rcmail extends rcube
         if (!$_SESSION['user_id']) {
             $_SESSION['temp'] = true;
         }
-
-        // restore skin selection after logout
-        if ($_SESSION['temp'] && !empty($_SESSION['skin'])) {
-            $this->config->set('skin', $_SESSION['skin']);
-        }
     }
 
     /**
@@ -606,8 +606,10 @@ class rcmail extends rcube
 
         // Here we need IDNA ASCII
         // Only rcube_contacts class is using domain names in Unicode
-        $host     = rcube_utils::idn_to_ascii($host);
-        $username = rcube_utils::idn_to_ascii($username);
+        $host = rcube_utils::idn_to_ascii($host);
+        if (strpos($username, '@')) {
+            $username = rcube_utils::idn_to_ascii($username);
+        }
 
         // user already registered -> overwrite username
         if ($user = rcube_user::query($username, $host)) {
@@ -678,8 +680,9 @@ class rcmail extends rcube
             $_SESSION['password']     = $this->encrypt($password);
             $_SESSION['login_time']   = time();
 
-            if (isset($_REQUEST['_timezone']) && $_REQUEST['_timezone'] != '_default_') {
-                $_SESSION['timezone'] = rcube_utils::get_input_value('_timezone', rcube_utils::INPUT_GPC);
+            $timezone = rcube_utils::get_input_value('_timezone', rcube_utils::INPUT_GPC);
+            if ($timezone && is_string($timezone) && $timezone != '_default_') {
+                $_SESSION['timezone'] = $timezone;
             }
 
             // fix some old settings according to namespace prefix
@@ -713,6 +716,40 @@ class rcmail extends rcube
         }
     }
 
+    /**
+     * Detects session errors
+     *
+     * @return string Error label
+     */
+    public function session_error()
+    {
+        // log session failures
+        $task = rcube_utils::get_input_value('_task', rcube_utils::INPUT_GPC);
+
+        if ($task && !in_array($task, array('login', 'logout')) && ($sess_id = $_COOKIE[ini_get('session.name')])) {
+            $log   = "Aborted session $sess_id; no valid session data found";
+            $error = 'sessionerror';
+
+            // In rare cases web browser might end up with multiple cookies of the same name
+            // but different params, e.g. domain (webmail.domain.tld and .webmail.domain.tld).
+            // In such case browser will send both cookies in the request header
+            // problem is that PHP session handler can use only one and if that one session
+            // does not exist we'll end up here
+            $cookie          = rcube_utils::request_header('Cookie');
+            $cookie_sessid   = $this->config->get('session_name') ?: 'roundcube_sessid';
+            $cookie_sessauth = $this->config->get('session_auth_name') ?: 'roundcube_sessauth';
+
+            if (substr_count($cookie, $cookie_sessid.'=') > 1 || substr_count($cookie, $cookie_sessauth.'=') > 1) {
+                $log .= ". Cookies mismatch";
+                $error = 'cookiesmismatch';
+            }
+
+            $this->session->log($log);
+
+            return $error;
+        }
+    }
+
     /**
      * Auto-select IMAP host based on the posted login information
      *
@@ -771,8 +808,12 @@ class rcmail extends rcube
         $this->plugins->exec_hook('session_destroy');
 
         $this->session->kill();
-        $_SESSION = array('language' => $this->user->language, 'temp' => true, 'skin' => $this->config->get('skin'));
+        $_SESSION = array('language' => $this->user->language, 'temp' => true);
         $this->user->reset();
+
+        if ($this->config->get('skin') != $this->default_skin && method_exists($this->output, 'set_skin')) {
+            $this->output->set_skin($this->default_skin);
+        }
     }
 
     /**
@@ -887,14 +928,17 @@ class rcmail extends rcube
 
         // write performance stats to logs/console
         if ($this->config->get('devel_mode') || $this->config->get('performance_stats')) {
+            // we have to disable per_user_logging to make sure stats end up in the main console log
+            $this->config->set('per_user_logging', false);
+
             // make sure logged numbers use unified format
             setlocale(LC_NUMERIC, 'en_US.utf8', 'en_US.UTF-8', 'en_US', 'C');
 
             if (function_exists('memory_get_usage')) {
-                $mem = $this->show_bytes(memory_get_usage());
+                $mem = round(memory_get_usage() / 1024 /1024, 1);
             }
             if (function_exists('memory_get_peak_usage')) {
-                $mem .= '/'.$this->show_bytes(memory_get_peak_usage());
+                $mem .= '/'. round(memory_get_peak_usage() / 1024 / 1024, 1);
             }
 
             $log = $this->task . ($this->action ? '/'.$this->action : '') . ($mem ? " [$mem]" : '');
@@ -920,12 +964,6 @@ class rcmail extends rcube
             $error = array('code' => 403, 'message' => "Request security check failed");
             self::raise_error($error, false, true);
         }
-
-        // check referer if configured
-        if ($this->config->get('referer_check') && !rcube_utils::check_referer()) {
-            $error = array('code' => 403, 'message' => "Referer check failed");
-            self::raise_error($error, true, true);
-        }
     }
 
     /**
@@ -1107,6 +1145,10 @@ class rcmail extends rcube
             return;
         }
 
+        // don't log full session id for security reasons
+        $session_id = session_id();
+        $session_id = $session_id ? substr($session_id, 0, 16) : 'no-session';
+
         // failed login
         if ($failed_login) {
             // don't fill the log with complete input, which could
@@ -1116,7 +1158,7 @@ class rcmail extends rcube
             }
 
             $message = sprintf('Failed login for %s from %s in session %s (error: %d)',
-                $user, rcube_utils::remote_ip(), session_id(), $error_code);
+                $user, rcube_utils::remote_ip(), $session_id, $error_code);
         }
         // successful login
         else {
@@ -1128,13 +1170,45 @@ class rcmail extends rcube
             }
 
             $message = sprintf('Successful login for %s (ID: %d) from %s in session %s',
-                $user_name, $user_id, rcube_utils::remote_ip(), session_id());
+                $user_name, $user_id, rcube_utils::remote_ip(), $session_id);
         }
 
         // log login
         self::write_log('userlogins', $message);
     }
 
+    /**
+     * Check if specified asset file exists
+     *
+     * @param string $path     Asset path
+     * @param bool   $minified Fallback to minified version of the file
+     *
+     * @return string Asset path if found (modified if minified file found)
+     */
+    public function find_asset($path, $minified = true)
+    {
+        if (empty($path)) {
+            return;
+        }
+
+        $assets_dir = $this->config->get('assets_dir');
+        $root_path  = unslashify($assets_dir ?: INSTALL_PATH) . '/';
+        $full_path  = $root_path . trim($path, '/');
+
+        if (file_exists($full_path)) {
+            return $path;
+        }
+
+        if ($minified && preg_match('/(?<!\.min)\.(js|css)$/', $path)) {
+            $path      = preg_replace('/\.(js|css)$/', '.min.\\1', $path);
+            $full_path = $root_path . trim($path, '/');
+
+            if (file_exists($full_path)) {
+                return $path;
+            }
+        }
+    }
+
     /**
      * Create a HTML table based on the given data
      *
@@ -1459,6 +1533,16 @@ class rcmail extends rcube
             $this->build_folder_tree($a_mailboxes, $folder, $delimiter);
         }
 
+        // allow plugins to alter the folder tree or to localize folder names
+        $hook = $this->plugins->exec_hook('render_folder_selector', array(
+            'list'      => $a_mailboxes,
+            'delimiter' => $delimiter,
+            'attribs'   => $p,
+        ));
+
+        $a_mailboxes = $hook['list'];
+        $p           = $hook['attribs'];
+
         $select = new html_select($p);
 
         if ($p['noselection']) {
@@ -1583,7 +1667,7 @@ class rcmail extends rcube
             }
 
             $js_name     = $this->JQ($folder['id']);
-            $html_name   = $this->Q($foldername) . ($unread ? html::span('unreadcount', sprintf($attrib['unreadwrap'], $unread)) : '');
+            $html_name   = $this->Q($foldername) . ($unread ? html::span('unreadcount skip-content', sprintf($attrib['unreadwrap'], $unread)) : '');
             $link_attrib = $folder['virtual'] ? array() : array(
                 'href'    => $this->url(array('_mbox' => $folder['id'])),
                 'onclick' => sprintf("return %s.command('list','%s',this,event)", rcmail_output::JS_OBJECT_NAME, $js_name),
@@ -1593,7 +1677,7 @@ class rcmail extends rcube
 
             $out .= html::tag('li', array(
                     'id'      => "rcmli" . $folder_id,
-                    'class'   => join(' ', $classes),
+                    'class'   => implode(' ', $classes),
                     'noclose' => true
                 ),
                 html::a($link_attrib, $html_name));
@@ -1806,7 +1890,7 @@ class rcmail extends rcube
                 $quota_result['percent'] = min(100, round(($quota['used']/max(1,$quota['total']))*100));
             }
 
-            $title = sprintf('%s / %s (%.0f%%)',
+            $title = $this->gettext('quota') . ': ' . sprintf('%s / %s (%.0f%%)',
                 $this->show_bytes($quota['used'] * 1024),
                 $this->show_bytes($quota['total'] * 1024),
                 $quota_result['percent']
@@ -1930,6 +2014,22 @@ class rcmail extends rcube
         }
     }
 
+    /**
+     * Displays an error message on storage fatal errors
+     */
+    public function storage_fatal_error()
+    {
+        $err_code = $this->storage->get_error_code();
+        switch ($err_code) {
+        // Not all are really fatal, but these should catch
+        // connection/authentication errors the best we can
+        case rcube_imap_generic::ERROR_NO:
+        case rcube_imap_generic::ERROR_BAD:
+        case rcube_imap_generic::ERROR_BYE:
+            $this->display_server_error();
+        }
+    }
+
     /**
      * Output HTML editor scripts
      *
@@ -1962,6 +2062,7 @@ class rcmail extends rcube
 
         $lang_codes = array($_SESSION['language']);
         $assets_dir = $this->config->get('assets_dir') ?: INSTALL_PATH;
+        $skin_path  = $this->output->get_skin_path();
 
         if ($pos = strpos($_SESSION['language'], '_')) {
             $lang_codes[] = substr($_SESSION['language'], 0, $pos);
@@ -1981,94 +2082,43 @@ class rcmail extends rcube
         $config = array(
             'mode'       => $mode,
             'lang'       => $lang,
-            'skin_path'  => $this->output->get_skin_path(),
+            'skin_path'  => $skin_path,
             'spellcheck' => $spellcheck, // deprecated
             'spelldict'  => $spelldict,
+            'content_css'      => 'program/resources/tinymce/content.css',
             'disabled_plugins' => $hook['disabled_plugins'],
             'disabled_buttons' => $hook['disabled_buttons'],
             'extra_plugins'    => $hook['extra_plugins'],
             'extra_buttons'    => $hook['extra_buttons'],
         );
 
+        if ($path = $this->config->get('editor_css_location')) {
+            if ($path = $this->find_asset($skin_path . $path)) {
+                $config['content_css'] = $path;
+            }
+        }
+
         $this->output->add_label('selectimage', 'addimage', 'selectmedia', 'addmedia');
         $this->output->set_env('editor_config', $config);
-        $this->output->include_css('program/resources/tinymce/browser.css');
+
+        if ($path = $this->config->get('media_browser_css_location', 'program/resources/tinymce/browser.css')) {
+            if ($path != 'none' && ($path = $this->find_asset($path))) {
+                $this->output->include_css($path);
+            }
+        }
+
         $this->output->include_script('tinymce/tinymce.min.js');
         $this->output->include_script('editor.js');
     }
 
     /**
      * File upload progress handler.
+     *
+     * @deprecated We're using HTML5 upload progress
      */
     public function upload_progress()
     {
-        $params = array(
-            'action' => $this->action,
-            'name'   => rcube_utils::get_input_value('_progress', rcube_utils::INPUT_GET),
-        );
-
-        if (function_exists('uploadprogress_get_info')) {
-            $status = uploadprogress_get_info($params['name']);
-
-            if (!empty($status)) {
-                $params['current'] = $status['bytes_uploaded'];
-                $params['total']   = $status['bytes_total'];
-            }
-        }
-
-        if (!isset($status) && filter_var(ini_get('apc.rfc1867'), FILTER_VALIDATE_BOOLEAN)
-            && ini_get('apc.rfc1867_name')
-        ) {
-            $prefix = ini_get('apc.rfc1867_prefix');
-            $status = apc_fetch($prefix . $params['name']);
-
-            if (!empty($status)) {
-                $params['current'] = $status['current'];
-                $params['total']   = $status['total'];
-            }
-        }
-
-        if (!isset($status) && filter_var(ini_get('session.upload_progress.enabled'), FILTER_VALIDATE_BOOLEAN)
-            && ini_get('session.upload_progress.name')
-        ) {
-            $key = ini_get('session.upload_progress.prefix') . $params['name'];
-
-            $params['total']   = $_SESSION[$key]['content_length'];
-            $params['current'] = $_SESSION[$key]['bytes_processed'];
-        }
-
-        if (!empty($params['total'])) {
-            $total = $this->show_bytes($params['total'], $unit);
-            switch ($unit) {
-            case 'GB':
-                $gb      = $params['current']/1073741824;
-                $current = sprintf($gb >= 10 ? "%d" : "%.1f", $gb);
-                break;
-            case 'MB':
-                $mb      = $params['current']/1048576;
-                $current = sprintf($mb >= 10 ? "%d" : "%.1f", $mb);
-                break;
-            case 'KB':
-                $current = round($params['current']/1024);
-                break;
-            case 'B':
-            default:
-                $current = $params['current'];
-                break;
-            }
-
-            $params['percent'] = round($params['current']/$params['total']*100);
-            $params['text']    = $this->gettext(array(
-                'name' => 'uploadprogress',
-                'vars' => array(
-                    'percent' => $params['percent'] . '%',
-                    'current' => $current,
-                    'total'   => $total
-                )
-            ));
-        }
-
-        $this->output->command('upload_progress_update', $params);
+        // NOOP
         $this->output->send();
     }
 
@@ -2081,24 +2131,6 @@ class rcmail extends rcube
      */
     public function upload_init($max_size = null)
     {
-        // Enable upload progress bar
-        if ($seconds = $this->config->get('upload_progress')) {
-            if (function_exists('uploadprogress_get_info')) {
-                $field_name = 'UPLOAD_IDENTIFIER';
-            }
-            if (!$field_name && filter_var(ini_get('apc.rfc1867'), FILTER_VALIDATE_BOOLEAN)) {
-                $field_name = ini_get('apc.rfc1867_name');
-            }
-            if (!$field_name && filter_var(ini_get('session.upload_progress.enabled'), FILTER_VALIDATE_BOOLEAN)) {
-                $field_name = ini_get('session.upload_progress.name');
-            }
-
-            if ($field_name) {
-                $this->output->set_env('upload_progress_name', $field_name);
-                $this->output->set_env('upload_progress_time', (int) $seconds);
-            }
-        }
-
         // find max filesize value
         $max_filesize = rcube_utils::max_upload_size();
         if ($max_size && $max_size < $max_filesize) {
@@ -2116,6 +2148,8 @@ class rcmail extends rcube
                 'name' => 'filecounterror', 'vars' => array('count' => $max_filecount))));
         }
 
+        $this->output->add_label('uploadprogress', 'GB', 'MB', 'KB', 'B');
+
         return $max_filesize_txt;
     }
 
@@ -2176,8 +2210,7 @@ class rcmail extends rcube
         $content = $attrib['prefix'] . $input->show();
 
         if ($attrib['mode'] != 'smart') {
-            $content  = html::div(null, $content);
-            $content .= $hint;
+            $content = html::div(null, $content . $hint);
         }
 
         if (rcube_utils::get_boolean($attrib['buttons'])) {
@@ -2213,17 +2246,16 @@ class rcmail extends rcube
 
             // generate image thumbnail for file browser in HTML editor
             if (!empty($_GET['_thumbnail'])) {
-                $temp_dir       = $this->config->get('temp_dir');
                 $thumbnail_size = 80;
                 $mimetype       = $file['mimetype'];
                 $file_ident     = $file['id'] . ':' . $file['mimetype'] . ':' . $file['size'];
-                $cache_basename = $temp_dir . '/' . md5($file_ident . ':' . $this->user->ID . ':' . $thumbnail_size);
-                $cache_file     = $cache_basename . '.thumb';
+                $thumb_name     = 'thumb' . md5($file_ident . ':' . $this->user->ID . ':' . $thumbnail_size);
+                $cache_file     = rcube_utils::temp_filename($thumb_name, false, false);
 
                 // render thumbnail image if not done yet
                 if (!is_file($cache_file)) {
                     if (!$file['path']) {
-                        $orig_name = $filename = $cache_basename . '.tmp';
+                        $orig_name = $filename = $cache_file . '.tmp';
                         file_put_contents($orig_name, $file['data']);
                     }
                     else {
@@ -2331,6 +2363,15 @@ class rcmail extends rcube
      */
     public function show_bytes($bytes, &$unit = null)
     {
+        // Plugins may want to display different units
+        $plugin = $this->plugins->exec_hook('show_bytes', array('bytes' => $bytes));
+
+        $unit = $plugin['unit'];
+
+        if ($plugin['result'] !== null) {
+            return $plugin['result'];
+        }
+
         if ($bytes >= 1073741824) {
             $unit = 'GB';
             $gb   = $bytes/1073741824;
@@ -2392,16 +2433,17 @@ class rcmail extends rcube
      * @param string $uids           UID value to decode
      * @param string $mbox           Default mailbox value (if not encoded in UIDs)
      * @param bool   $is_multifolder Will be set to True if multi-folder request
+     * @param int    $mode           Request mode. Default: rcube_utils::INPUT_GPC.
      *
      * @return array  List of message UIDs per folder
      */
-    public static function get_uids($uids = null, $mbox = null, &$is_multifolder = false)
+    public static function get_uids($uids = null, $mbox = null, &$is_multifolder = false, $mode = null)
     {
         // message UID (or comma-separated list of IDs) is provided in
         // the form of <ID>-<MBOX>[,<ID>-<MBOX>]*
 
-        $_uid  = $uids ?: rcube_utils::get_input_value('_uid', rcube_utils::INPUT_GPC);
-        $_mbox = $mbox ?: (string) rcube_utils::get_input_value('_mbox', rcube_utils::INPUT_GPC);
+        $_uid  = $uids ?: rcube_utils::get_input_value('_uid', $mode ?: rcube_utils::INPUT_GPC);
+        $_mbox = $mbox ?: (string) rcube_utils::get_input_value('_mbox', $mode ?: rcube_utils::INPUT_GPC);
 
         // already a hash array
         if (is_array($_uid) && !isset($_uid[0])) {
@@ -2420,8 +2462,9 @@ class rcmail extends rcube
             }
         }
         else {
-            if (is_string($_uid))
+            if (is_string($_uid)) {
                 $_uid = explode(',', $_uid);
+            }
 
             // create a per-folder UIDs array
             foreach ((array)$_uid as $uid) {
@@ -2436,7 +2479,7 @@ class rcmail extends rcube
                 if ($uid == '*') {
                     $result[$mbox] = $uid;
                 }
-                else {
+                else if (preg_match('/^[0-9:.]+$/', $uid)) {
                     $result[$mbox][] = $uid;
                 }
             }
