@@ -3,7 +3,8 @@
 /**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2005-2013, The Roundcube Dev Team                       |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -32,8 +33,8 @@ class html
     public static $doctype = 'xhtml';
     public static $lc_tags = true;
     public static $common_attrib = array('id','class','style','title','align','unselectable','tabindex','role');
-    public static $containers    = array('iframe','div','span','p','h1','h2','h3','ul','form','textarea','table','thead','tbody','tr','th','td','style','script');
-    public static $bool_attrib   = array('checked','multiple','disabled','selected','autofocus','readonly');
+    public static $containers    = array('iframe','div','span','p','h1','h2','h3','ul','form','textarea','table','thead','tbody','tr','th','td','style','script','a');
+    public static $bool_attrib   = array('checked','multiple','disabled','selected','autofocus','readonly','required');
 
 
     /**
@@ -65,7 +66,7 @@ class html
      *
      * @param string $tagname Tag name
      * @param array  $attrib  Tag attributes as key/value pairs
-     * @param string $content Optinal Tag content (creates a container tag)
+     * @param string $content Optional Tag content (creates a container tag)
      * @param array  $allowed List with allowed attributes, omit to allow all
      *
      * @return string The XHTML tag
@@ -163,7 +164,7 @@ class html
         }
 
         return self::tag('img', $attr + array('alt' => ''), null, array_merge(self::$common_attrib,
-            array('src','alt','width','height','border','usemap','onclick','onerror')));
+            array('src','alt','width','height','border','usemap','onclick','onerror','onload')));
     }
 
     /**
@@ -218,13 +219,15 @@ class html
             $attr = array('for' => $attr);
         }
 
-        return self::tag('label', $attr, $cont, array_merge(self::$common_attrib, array('for')));
+        return self::tag('label', $attr, $cont, array_merge(self::$common_attrib,
+            array('for','onkeypress')));
     }
 
     /**
      * Derrived method to create <iframe></iframe>
      *
-     * @param mixed $attr Hash array with tag attributes or string with frame source (src)
+     * @param mixed  $attr Hash array with tag attributes or string with frame source (src)
+     * @param string $cont Tag content
      *
      * @return string HTML code
      * @see html::tag()
@@ -267,7 +270,7 @@ class html
     /**
      * Derrived method for line breaks
      *
-     * @param array $attrib  Associative arry with tag attributes
+     * @param array $attrib Associative arry with tag attributes
      *
      * @return string HTML code
      * @see html::tag()
@@ -322,8 +325,12 @@ class html
             // attributes with no value
             if (in_array($key, self::$bool_attrib)) {
                 if ($value) {
-                    // @TODO: minimize attribute in non-xhtml mode
-                    $attrib_arr[] = $key . '="' . $key . '"';
+                    $value = $key;
+                    if (self::$doctype == 'xhtml') {
+                        $value .= '="' . $value . '"';
+                    }
+
+                    $attrib_arr[] = $value;
                 }
             }
             else {
@@ -337,21 +344,24 @@ class html
     /**
      * Convert a HTML attribute string attributes to an associative array (name => value)
      *
-     * @param string Input string
+     * @param string $str Input string
      *
      * @return array Key-value pairs of parsed attributes
      */
     public static function parse_attrib_string($str)
     {
         $attrib = array();
-        $regexp = '/\s*([-_a-z]+)=(["\'])??(?(2)([^\2]*)\2|(\S+?))/Ui';
+        $html   = '<html>'
+            . '<head><meta http-equiv="Content-Type" content="text/html; charset=' . RCUBE_CHARSET . '" /></head>'
+            . '<body><div ' . rtrim($str, '/ ') . ' /></body>'
+            . '</html>';
 
-        preg_match_all($regexp, stripslashes($str), $regs, PREG_SET_ORDER);
+        $document = new DOMDocument('1.0', RCUBE_CHARSET);
+        @$document->loadHTML($html);
 
-        // convert attributes to an associative array (name => value)
-        if ($regs) {
-            foreach ($regs as $attr) {
-                $attrib[strtolower($attr[1])] = html_entity_decode($attr[3] . $attr[4]);
+        if ($node = $document->getElementsByTagName('div')->item(0)) {
+            foreach ($node->attributes as $name => $attr) {
+                $attrib[strtolower($name)] = $attr->nodeValue;
             }
         }
 
@@ -367,16 +377,7 @@ class html
      */
     public static function quote($str)
     {
-        static $flags;
-
-        if (!$flags) {
-            $flags = ENT_COMPAT;
-            if (defined('ENT_SUBSTITUTE')) {
-                $flags |= ENT_SUBSTITUTE;
-            }
-        }
-
-        return @htmlspecialchars($str, $flags, RCUBE_CHARSET);
+        return @htmlspecialchars($str, ENT_COMPAT | ENT_SUBSTITUTE, RCUBE_CHARSET);
     }
 }
 
@@ -395,7 +396,7 @@ class html_inputfield extends html
         'type','name','value','size','tabindex','autocapitalize','required',
         'autocomplete','checked','onchange','onclick','disabled','readonly',
         'spellcheck','results','maxlength','src','multiple','accept',
-        'placeholder','autofocus','pattern'
+        'placeholder','autofocus','pattern','oninput'
     );
 
     /**
@@ -566,6 +567,38 @@ class html_checkbox extends html_inputfield
     }
 }
 
+/**
+ * Class to create HTML button
+ *
+ * @package    Framework
+ * @subpackage View
+ */
+class html_button extends html_inputfield
+{
+    protected $tagname = 'button';
+    protected $type    = 'button';
+
+    /**
+     * Get HTML code for this object
+     *
+     * @param string $content Text Content of the button
+     * @param array  $attrib  Additional attributes to override
+     *
+     * @return string HTML output
+     */
+    public function show($content = '', $attrib = null)
+    {
+        // overwrite object attributes
+        if (is_array($attrib)) {
+            $this->attrib = array_merge($this->attrib, $attrib);
+        }
+
+        $this->content = $content;
+
+        return parent::show();
+    }
+}
+
 /**
  * Class to create an HTML textarea
  *
@@ -625,7 +658,7 @@ class html_textarea extends html
  * $select->add(array('Switzerland','Germany'), array('CH','DE'));
  *
  * // generate pulldown with selection 'Switzerland'  and return html-code
- * // as second argument the same attributes available to instanciate can be used
+ * // as second argument the same attributes available to instantiate can be used
  * print $select->show('CH');
  * </pre>
  *
@@ -743,6 +776,10 @@ class html_table extends html
         $cell->attrib  = $attr;
         $cell->content = $cont;
 
+        if (!isset($this->rows[$this->rowindex])) {
+            $this->rows[$this->rowindex] = new stdClass;
+        }
+
         $this->rows[$this->rowindex]->cells[$this->colindex] = $cell;
         $this->colindex += max(1, intval($attr['colspan']));
 
@@ -773,12 +810,12 @@ class html_table extends html
      * Remove a column from a table
      * Useful for plugins making alterations
      *
-     * @param string $class
+     * @param string $class Class name
      */
     public function remove_column($class)
     {
         // Remove the header
-        foreach ($this->header as $index=>$header){
+        foreach ($this->header as $index => $header){
             if ($header->attrib['class'] == $class){
                 unset($this->header[$index]);
                 break;
@@ -786,8 +823,8 @@ class html_table extends html
         }
 
         // Remove cells from rows
-        foreach ($this->rows as $i=>$row){
-            foreach ($row->cells as $j=>$cell){
+        foreach ($this->rows as $i => $row){
+            foreach ($row->cells as $j => $cell){
                 if ($cell->attrib['class'] == $class){
                     unset($this->rows[$i]->cells[$j]);
                     break;
@@ -863,26 +900,35 @@ class html_table extends html
             $this->attrib = array_merge($this->attrib, $attrib);
         }
 
-        $thead = $tbody = "";
+        $thead        = '';
+        $tbody        = '';
+        $col_tagname  = $this->_col_tagname();
+        $row_tagname  = $this->_row_tagname();
+        $head_tagname = $this->_head_tagname();
 
         // include <thead>
         if (!empty($this->header)) {
             $rowcontent = '';
             foreach ($this->header as $c => $col) {
-                $rowcontent .= self::tag($this->_head_tagname(), $col->attrib, $col->content);
+                $rowcontent .= self::tag($head_tagname, $col->attrib, $col->content);
             }
             $thead = $this->tagname == 'table' ? self::tag('thead', null, self::tag('tr', null, $rowcontent, parent::$common_attrib)) :
-                self::tag($this->_row_tagname(), array('class' => 'thead'), $rowcontent, parent::$common_attrib);
+                self::tag($row_tagname, array('class' => 'thead'), $rowcontent, parent::$common_attrib);
         }
 
         foreach ($this->rows as $r => $row) {
             $rowcontent = '';
             foreach ($row->cells as $c => $col) {
-                $rowcontent .= self::tag($this->_col_tagname(), $col->attrib, $col->content);
+                if ($row_tagname == 'li' && empty($col->attrib) && count($row->cells) == 1) {
+                    $rowcontent .= $col->content;
+                }
+                else {
+                    $rowcontent .= self::tag($col_tagname, $col->attrib, $col->content);
+                }
             }
 
             if ($r < $this->rowindex || count($row->cells)) {
-                $tbody .= self::tag($this->_row_tagname(), $row->attrib, $rowcontent, parent::$common_attrib);
+                $tbody .= self::tag($row_tagname, $row->attrib, $rowcontent, parent::$common_attrib);
             }
         }
 
@@ -894,13 +940,14 @@ class html_table extends html
         $this->content = $thead . ($this->tagname == 'table' ? self::tag('tbody', null, $tbody) : $tbody);
 
         unset($this->attrib['cols'], $this->attrib['rowsonly']);
+
         return parent::show();
     }
 
     /**
      * Count number of rows
      *
-     * @return The number of rows
+     * @return int The number of rows
      */
     public function size()
     {
