@@ -3,7 +3,8 @@
 /**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2008-2014, The Roundcube Dev Team                       |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -24,13 +25,15 @@
  */
 class rcube_config
 {
-    const DEFAULT_SKIN = 'larry';
+    const DEFAULT_SKIN = 'elastic';
 
     private $env       = '';
     private $paths     = array();
     private $prop      = array();
     private $errors    = array();
     private $userprefs = array();
+    private $immutable = array();
+    private $client_tz;
 
 
     /**
@@ -47,6 +50,7 @@ class rcube_config
         'min_refresh_interval' => 'min_keep_alive',
         'messages_cache_ttl'   => 'message_cache_lifetime',
         'mail_read_time'       => 'preview_pane_mark_read',
+        'session_debug'        => 'log_session',
         'redundant_attachments_cache_ttl' => 'redundant_attachments_memcache_ttl',
     );
 
@@ -191,8 +195,6 @@ class rcube_config
 
     /**
      * Load config from local config file
-     *
-     * @todo Remove global $CONFIG
      */
     private function load()
     {
@@ -225,7 +227,6 @@ class rcube_config
             }
         }
 
-        // larry is the new default skin :-)
         if ($this->prop['skin'] == 'default') {
             $this->prop['skin'] = self::DEFAULT_SKIN;
         }
@@ -246,25 +247,18 @@ class rcube_config
         }
 
         // set PHP error logging according to config
-        if ($this->prop['debug_level'] & 1) {
-            ini_set('log_errors', 1);
-
-            if ($this->prop['log_driver'] == 'syslog') {
-                ini_set('error_log', 'syslog');
-            }
-            else {
-                ini_set('error_log', $this->prop['log_dir'].'/errors');
-            }
+        $error_log = $this->prop['log_driver'] ?: 'file';
+        if ($error_log == 'file') {
+            $error_log  = $this->prop['log_dir'] . '/errors';
+            $error_log .= isset($this->prop['log_file_ext']) ? $this->prop['log_file_ext'] : '.log';
         }
 
-        // enable display_errors in 'show' level, but not for ajax requests
-        ini_set('display_errors', intval(empty($_REQUEST['_remote']) && ($this->prop['debug_level'] & 4)));
+        if ($error_log && $error_log != 'stdout') {
+            ini_set('error_log', $error_log);
+        }
 
         // remove deprecated properties
         unset($this->prop['dst_active']);
-
-        // export config data
-        $GLOBALS['CONFIG'] = &$this->prop;
     }
 
     /**
@@ -415,12 +409,17 @@ class rcube_config
     /**
      * Setter for a config parameter
      *
-     * @param string $name  Parameter name
-     * @param mixed  $value Parameter value
+     * @param string $name      Parameter name
+     * @param mixed  $value     Parameter value
+     * @param bool   $immutable Make the value immutable
      */
-    public function set($name, $value)
+    public function set($name, $value, $immutable = false)
     {
         $this->prop[$name] = $value;
+
+        if ($immutable) {
+            $this->immutable[$name] = $value;
+        }
     }
 
     /**
@@ -431,7 +430,7 @@ class rcube_config
     public function merge($prefs)
     {
         $prefs = $this->fix_legacy_props($prefs);
-        $this->prop = array_merge($this->prop, $prefs, $this->userprefs);
+        $this->prop = array_merge($this->prop, $prefs, $this->userprefs, $this->immutable);
     }
 
     /**
@@ -452,7 +451,6 @@ class rcube_config
             }
         }
 
-        // larry is the new default skin :-)
         if ($prefs['skin'] == 'default') {
             $prefs['skin'] = self::DEFAULT_SKIN;
         }
@@ -481,6 +479,18 @@ class rcube_config
         return $plugin['result'];
     }
 
+    /**
+     * Some options set as immutable that are also listed
+     * in dont_override should not be stored permanently
+     * in user preferences. Here's the list of these
+     *
+     * @return array List of transient options
+     */
+    public function transient_options()
+    {
+        return array_intersect(array_keys($this->immutable), (array) $this->get('dont_override'));
+    }
+
     /**
      * Special getter for user's timezone offset including DST
      *
@@ -566,6 +576,26 @@ class rcube_config
         return "\n";
     }
 
+    /**
+     * Returns list of configured PGP key servers
+     *
+     * @return array|null List of keyservers' URLs
+     */
+    public function keyservers()
+    {
+        $list = (array) $this->prop['keyservers'];
+
+        foreach ($list as $idx => $host) {
+            if (!preg_match('|^[a-z]://|', $host)) {
+                $host = "https://$host";
+            }
+
+            $list[$idx] = slashify($host);
+        }
+
+        return !empty($list) ? $list : null;
+    }
+
     /**
      * Return the mail domain configured for the given host
      *
@@ -601,7 +631,7 @@ class rcube_config
      */
     public function get_error()
     {
-        return empty($this->errors) ? false : join("\n", $this->errors);
+        return empty($this->errors) ? false : implode("\n", $this->errors);
     }
 
     /**
@@ -609,13 +639,20 @@ class rcube_config
      */
     private function client_timezone()
     {
+        if ($this->client_tz) {
+            return $this->client_tz;
+        }
+
         // @TODO: remove this legacy timezone handling in the future
         $props = $this->fix_legacy_props(array('timezone' => $_SESSION['timezone']));
 
         if (!empty($props['timezone'])) {
+            // Prevent from using deprecated timezone names
+            $props['timezone'] = $this->resolve_timezone_alias($props['timezone']);
+
             try {
                 $tz = new DateTimeZone($props['timezone']);
-                return $tz->getName();
+                return $this->client_tz = $tz->getName();
             }
             catch (Exception $e) { /* gracefully ignore */ }
         }
@@ -658,6 +695,12 @@ class rcube_config
             unset($props['preview_pane']);
         }
 
+        // translate old `display_version` settings to `display_product_info`
+        if (isset($props['display_version']) && !isset($props['display_product_info'])) {
+            $props['display_product_info'] = $props['display_version'] ? 2 : 1;
+            unset($props['display_version']);
+        }
+
         return $props;
     }
 
@@ -720,4 +763,142 @@ class rcube_config
 
         return $timezones[(string) intval($offset * 60)];
     }
+
+    /**
+     * Replace deprecated timezone name with a valid one.
+     *
+     * @param string $tzname Timezone name
+     *
+     * @return string Timezone name
+     */
+    static public function resolve_timezone_alias($tzname)
+    {
+        // http://www.php.net/manual/en/timezones.others.php
+        // https://en.wikipedia.org/wiki/List_of_tz_database_time_zones
+        $deprecated_timezones = array(
+            'Australia/ACT'         => 'Australia/Sydney',
+            'Australia/LHI'         => 'Australia/Lord_Howe',
+            'Australia/North'       => 'Australia/Darwin',
+            'Australia/NSW'         => 'Australia/Sydney',
+            'Australia/Queensland'  => 'Australia/Brisbane',
+            'Australia/South'       => 'Australia/Adelaide',
+            'Australia/Adelaide'    => 'Australia/Hobart',
+            'Australia/Tasmania'    => 'Australia/Hobart',
+            'Australia/Victoria'    => 'Australia/Melbourne',
+            'Australia/West'        => 'Australia/Perth',
+            'Brazil/Acre'           => 'America/Rio_Branco',
+            'Brazil/DeNoronha'      => 'America/Noronha',
+            'Brazil/East'           => 'America/Sao_Paulo',
+            'Brazil/West'           => 'America/Manaus',
+            'Canada/Atlantic'       => 'America/Halifax',
+            'Canada/Central'        => 'America/Winnipeg',
+            'Canada/Eastern'        => 'America/Toronto',
+            'Canada/Mountain'       => 'America/Edmonton',
+            'Canada/Newfoundland'   => 'America/St_Johns',
+            'Canada/Pacific'        => 'America/Vancouver',
+            'Canada/Saskatchewan'   => 'America/Regina',
+            'Canada/Yukon'          => 'America/Whitehorse',
+            'CET'                   => 'Europe/Berlin',
+            'Chile/Continental'     => 'America/Santiago',
+            'Chile/EasterIsland'    => 'Pacific/Easter',
+            'CST6CDT'               => 'America/Chicago',
+            'Cuba'                  => ' America/Havana',
+            'EET'                   => 'Europe/Berlin',
+            'Egypt'                 => 'Africa/Cairo',
+            'Eire'                  => 'Europe/Dublin',
+            'EST'                   => 'America/New_York',
+            'EST5EDT'               => 'America/New_York',
+            'Factory'               => 'UTC', // ?
+            'GB'                    => 'Europe/London',
+            'GB-Eire'               => 'Europe/London',
+            'GMT'                   => 'UTC',
+            'GMT+0'                 => 'UTC',
+            'GMT-0'                 => 'UTC',
+            'GMT0'                  => 'UTC',
+            'Greenwich'             => 'UTC',
+            'Hongkong'              => 'Asia/Hong_Kong',
+            'HST'                   => 'Pacific/Honolulu',
+            'Iceland'               => 'Atlantic/Reykjavik',
+            'Iran'                  => 'Asia/Tehran',
+            'Israel'                => 'Asia/Jerusalem',
+            'Jamaica'               => 'America/Jamaica',
+            'Japan'                 => 'Asia/Tokyo',
+            'Kwajalein'             => 'Pacific/Kwajalein',
+            'Libya'                 => 'Africa/Tripoli',
+            'MET'                   => 'Europe/Berlin',
+            'Mexico/BajaNorte'      => 'America/Tijuana',
+            'Mexico/BajaSur'        => 'America/Mazatlan',
+            'Mexico/General'        => 'America/Mexico_City',
+            'MST'                   => 'America/Denver',
+            'MST7MDT'               => 'America/Denver',
+            'Navajo'                => 'America/Denver',
+            'NZ'                    => 'Pacific/Auckland',
+            'NZ-CHAT'               => 'Pacific/Chatham',
+            'Poland'                => 'Europe/Warsaw',
+            'Portugal'              => 'Europe/Lisbon',
+            'PRC'                   => 'Asia/Shanghai',
+            'PST8PDT'               => 'America/Los_Angeles',
+            'ROC'                   => 'Asia/Taipei',
+            'ROK'                   => 'Asia/Seoul',
+            'Singapore'             => 'Asia/Singapore',
+            'Turkey'                => 'Europe/Istanbul',
+            'UCT'                   => 'UTC',
+            'Universal'             => 'UTC',
+            'US/Alaska'             => 'America/Anchorage',
+            'US/Aleutian'           => 'America/Adak',
+            'US/Arizona'            => 'America/Phoenix',
+            'US/Central'            => 'America/Chicago',
+            'US/East-Indiana'       => 'America/Indiana/Indianapolis',
+            'US/Eastern'            => 'America/New_York',
+            'US/Hawaii'             => 'Pacific/Honolulu',
+            'US/Indiana-Starke'     => 'America/Indiana/Knox',
+            'US/Michigan'           => 'America/Detroit',
+            'US/Mountain'           => 'America/Denver',
+            'US/Pacific'            => 'America/Los_Angeles',
+            'US/Pacific-New'        => 'America/Los_Angeles',
+            'US/Samoa'              => 'Pacific/Pago_Pago',
+            'W-SU'                  => 'Europe/Moscow',
+            'WET'                   => 'Europe/Berlin',
+            'Z'                     => 'UTC',
+            'Zulu'                  => 'UTC',
+            // Some of these Etc/X zones are not deprecated, but still problematic
+            'Etc/GMT'           => 'UTC',
+            'Etc/GMT+0'         => 'UTC',
+            'Etc/GMT+1'         => 'Atlantic/Azores',
+            'Etc/GMT+10'        => 'Pacific/Honolulu',
+            'Etc/GMT+11'        => 'Pacific/Midway',
+            'Etc/GMT+12'        => 'Pacific/Auckland',
+            'Etc/GMT+2'         => 'America/Noronha',
+            'Etc/GMT+3'         => 'America/Argentina/Buenos_Aires',
+            'Etc/GMT+4'         => 'America/Manaus',
+            'Etc/GMT+5'         => 'America/New_York',
+            'Etc/GMT+6'         => 'America/Chicago',
+            'Etc/GMT+7'         => 'America/Denver',
+            'Etc/GMT+8'         => 'America/Los_Angeles',
+            'Etc/GMT+9'         => 'America/Anchorage',
+            'Etc/GMT-0'         => 'UTC',
+            'Etc/GMT-1'         => 'Europe/Berlin',
+            'Etc/GMT-10'        => 'Australia/Sydney',
+            'Etc/GMT-11'        => 'Pacific/Norfolk',
+            'Etc/GMT-12'        => 'Pacific/Auckland',
+            'Etc/GMT-13'        => 'Pacific/Apia',
+            'Etc/GMT-14'        => 'Pacific/Kiritimati',
+            'Etc/GMT-2'         => 'Africa/Cairo',
+            'Etc/GMT-3'         => 'Europe/Moscow',
+            'Etc/GMT-4'         => 'Europe/Samara',
+            'Etc/GMT-5'         => 'Asia/Yekaterinburg',
+            'Etc/GMT-6'         => 'Asia/Almaty',
+            'Etc/GMT-7'         => 'Asia/Bangkok',
+            'Etc/GMT-8'         => 'Asia/Hong_Kong',
+            'Etc/GMT-9'         => 'Asia/Tokyo',
+            'Etc/GMT0'          => 'UTC',
+            'Etc/Greenwich'     => 'UTC',
+            'Etc/UCT'           => 'UTC',
+            'Etc/Universal'     => 'UTC',
+            'Etc/UTC'           => 'UTC',
+            'Etc/Zulu'          => 'UTC',
+        );
+
+        return $deprecated_timezones[$tzname] ?: $tzname;
+    }
 }
