@@ -3,7 +3,8 @@
 /**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2005-2012, The Roundcube Dev Team                       |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -135,7 +136,6 @@ class rcube_db
 
         // connect to database
         if ($dbh = $this->conn_create($dsn)) {
-            $this->dbh          = $dbh;
             $this->dbhs[$mode]  = $dbh;
             $this->db_mode      = $mode;
             $this->db_connected = true;
@@ -160,12 +160,12 @@ class rcube_db
 
             $this->conn_prepare($dsn);
 
-            $dbh = new PDO($dsn_string, $dsn['username'], $dsn['password'], $dsn_options);
+            $this->dbh = new PDO($dsn_string, $dsn['username'], $dsn['password'], $dsn_options);
 
             // don't throw exceptions or warnings
-            $dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);
+            $this->dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_SILENT);
 
-            $this->conn_configure($dsn, $dbh);
+            $this->conn_configure($dsn, $this->dbh);
         }
         catch (Exception $e) {
             $this->db_error     = true;
@@ -178,7 +178,7 @@ class rcube_db
             return null;
         }
 
-        return $dbh;
+        return $this->dbh;
     }
 
     /**
@@ -914,7 +914,7 @@ class rcube_db
             $name[] = $start . $elem . $end;
         }
 
-        return implode($name, '.');
+        return implode('.', $name);
     }
 
     /**
@@ -1010,7 +1010,7 @@ class rcube_db
             $args = $args[0];
         }
 
-        return '(' . join(' || ', $args) . ')';
+        return '(' . implode(' || ', $args) . ')';
     }
 
     /**
